@@ -8,7 +8,13 @@ enum {
 	HR_HKEY_LOOKUP_FORWARD	= 2,
 	HR_HKEY_LOOKUP_REVERSE	= 3,
 	HR_HKEY_LOOKUP_KT		= 4,
-	HR_HKEY_LOOKUP_ATTR		= 5
+	HR_HKEY_LOOKUP_ATTR		= 5,
+    /*The following will probably never be used in C*/
+    HR_HKEY_LOOKUP_KEYFUNC  = 6,
+    HR_HKEY_LOOKUP_UNKEYFUNC= 7,
+    HR_HKEY_LOOKUP_FLAGS    = 8,
+    /*Used to cache stashes*/
+    HR_HKEY_LOOKUP_PRIVDATA = 9
 };
 
 
@@ -32,4 +38,11 @@ enum {
 #define HR_PKG_ATTR_SCALAR	"Ref::Store::XS::Attribute"
 #define HR_PKG_ATTR_ENCAP	"Ref::Store::XS::Attribute::Encapsulating"
 
-#endif /*HRDEFS_H_*/
\ No newline at end of file
+enum {
+    HR_STASH_KEY_SCALAR,
+    HR_STASH_KEY_ENCAP,
+    HR_STASH_ATTR_SCALAR,
+    HR_STASH_ATTR_ENCAP
+};
+
+#endif /*HRDEFS_H_*/
